/** @module recline/forms */

var recline = require('./index'),
	  error = require('./errors'),
		_ = require("underscore"),
		field_types = require('./dep/fields'),
		validations = require('./dep/validations'),
		template = require('./templates');

/**
	Sets the dbName, i.e "recline-forms"
	@param {string} string name of database
*/
exports.dbName = function(db) {
	this.dbname = db;
}

/**
	Sets up the form based on the form id
	It emits 'ready' event once the form 
	is ready to be manipulated
	
	@param {string} id id of the form
	@return {Forms} Form
*/
exports.get = function(id, cb){
	var self = this;
	if(typeof id != 'string') {
		throw new Error("id must be string");
	}
	
	var
			fClient = new Forms(),
		 	dbname = this.dbname || "forms",
		 	data,
			rev,
		 	db,
			fields,
			template;

	recline.createdb(dbname, function(err, ok){
		if(ok){
			db = recline[dbname];
			fClient.cb = cb;
			fClient.getOr404();
			fClient.template = template;
		}else{ cb && cb(err,null)}
	});
	
	fClient.__defineGetter__('id', function(){
		return id;
	});
	
	fClient.__defineGetter__('rev', function(){
		return rev;
	});
	
	fClient.__defineSetter__('rev', function(r){
		rev = r;
	});

	fClient.__defineGetter__('fields', function(){
		return fields;
	});
	
	fClient.__defineSetter__('fields', function(f){
		fields = f;
	});	


	fClient.__defineGetter__('data', function(){
		return data;
	});
	
	fClient.__defineSetter__('data', function(d){
		data = d;
	});
	
	fClient.__defineGetter__('db', function(){
		return db;
	});
	
	fClient.__defineSetter__('template', function(t){
		template = t;
	});
	
	fClient.__defineGetter__('template', function(){
		return template;
	});
	
	
	fClient._compileTemplate = function(type, stub){
		var compiled = _.template(field_types.lookup(type));
		return compiled(stub);
	}
	
	/**
		Returns args in a familar pattern
		return {object} $ callback, switch, id
		
	*/
	fClient._parseArgs = function(args){
		var $ = {};
		for(var key in args){
				// Annoying fix just for jsdoc
				if(_.isFunction(args[key])) $.cb = args[key];
				if(_.isBoolean(args[key])) 	$["switch"] = args[key];
				if(_.isString(args[key])) 	$.id = args[key];
		}
		return $;
	}
	
	return fClient;
	
}

/**
	The main forms Klass
	@class Forms
*/
var Forms = exports.Forms = function(){
}


/**
	Gets the form based on the id
	returns either the form or 404
	if it doesn't exist
*/
Forms.prototype.getOr404 = function(){
	var self = this;
	this.db.getDoc(this.id, function(er,res){
		if(er) { self.cb && _.bind(self.cb, this, 404,null)(); }
		else{ 
			self.data = res;
			self.rev = self.data._rev;			
			self.parseFields();
		}
	});
}

/**
	Parses an array of fields and complies them
	into template snippets for later rendering
	<br/>
	<b>It expects the array items to be objects:</b><br/>
	<p>
	<pre>
        "name": "email",
        "label": "Enter your email address",
        "type": "email",
        "primary": true,
        "required": true 
	</pre>
	</p>
	
	@param {array} fields array of fields
*/
Forms.prototype.parseFields = function(){
	var self = this;

	if(typeof this.fields === "undefined"){
		this.fields = [];
	}
	
	
	for(var key in self.data.fields){
		var type = self.data.fields[key].type;
		var stub = _.extend({ name: key }, _.clone(self.data.fields[key]));
		var field = [];
		
		field.push(this._compileTemplate("label", stub));
		

		if(type == "radios"){
			var l = self.data.fields[key].options.length;
			var radios = [];
			for(var i=0;i<l;i++){
				
				var stub = { name: key, value: self.data.fields[key].options[i], _id: key+"_"+i };
				radios.push(this._compileTemplate(type,stub));
				var stub = { name: key+"_"+i, label: self.data.fields[key].options[i] };
				radios.push(this._compileTemplate("label", stub));
				
			}
			field.push(radios);
		}
		else if(type == "select"){
			var l = self.data.fields[key].options.length;
			var select = [this._compileTemplate(type,stub)];
			for(var i=0;i<l;i++){
				
				var stub = { value: self.data.fields[key].options[i] };
				select.push(this._compileTemplate("option",stub));
				
			}
			select.push("</select>");
			field.push(select);
		}
			else
		{			
			field.push(this._compileTemplate(type,stub));
		}
		this.fields.push(field);
	}
	
	//console.log(this.fields);

	template.get(self.data.template, function(er,res){
		if(er) { self.cb && _.bind(self.cb, self, "error", null) }
		else{
			this.parseForm(self); 
		}
	})
		
}


